#!/usr/bin/env python
# 

from __future__ import print_function
import os, sys, re, copy, json, time, astropy
import numpy as np
from astropy.table import Table



# 
# data table directory
# 
data_table_directory = os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'Data_Tables')




# 
# def 
# 
def get_cosmic_mol_gas_density_A3COSMOS(method = 'using SIDES mock galaxy models'):
    # 
    # return z, rho_mol_gas
    # 
    if re.match(r'^.*\b(SIDES)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig15_plotted_data.txt')
    elif re.match(r'^.*\b(SMF)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig14_plotted_data.txt')
    else:
        raise Exception('Error! The \'method\' should be either "*SIDES*" or "*SMF*", but the input is "%s".'%(method))
    # 
    if not os.path.isfile(data_table_file):
        raise Exception('Error! Data file "%s" was not found!'%(data_table_file))
    # 
    tb = Table.read(data_table_file, format='ascii')
    z = copy.deepcopy(tb['z'].data)
    rho_mol_gas = copy.deepcopy(tb['rho_mol_gas_A3COSMOS'].data)
    # 
    return z, rho_mol_gas


def get_cosmic_mol_gas_density_Tacconi2018(method = 'using SIDES mock galaxy models'):
    # 
    # return z, rho_mol_gas
    # 
    if re.match(r'^.*\b(SIDES)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig15_plotted_data.txt')
    elif re.match(r'^.*\b(SMF)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig14_plotted_data.txt')
    else:
        raise Exception('Error! The \'method\' should be either "*SIDES*" or "*SMF*", but the input is "%s".'%(method))
    # 
    if not os.path.isfile(data_table_file):
        raise Exception('Error! Data file "%s" was not found!'%(data_table_file))
    # 
    tb = Table.read(data_table_file, format='ascii')
    z = copy.deepcopy(tb['z'].data)
    rho_mol_gas = copy.deepcopy(tb['rho_mol_gas_Tacconi2018'].data)
    # 
    return z, rho_mol_gas


def get_cosmic_mol_gas_density_Scoville2017(method = 'using SIDES mock galaxy models'):
    # 
    # return z, rho_mol_gas
    # 
    if re.match(r'^.*\b(SIDES)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig15_plotted_data.txt')
    elif re.match(r'^.*\b(SMF)\b.*$', method):
        data_table_file = os.path.join(data_table_directory, 'Liu2019b_Fig14_plotted_data.txt')
    else:
        raise Exception('Error! The \'method\' should be either "*SIDES*" or "*SMF*", but the input is "%s".'%(method))
    # 
    if not os.path.isfile(data_table_file):
        raise Exception('Error! Data file "%s" was not found!'%(data_table_file))
    # 
    tb = Table.read(data_table_file, format='ascii')
    z = copy.deepcopy(tb['z'].data)
    rho_mol_gas = copy.deepcopy(tb['rho_mol_gas_Scoville2017'].data)
    # 
    return z, rho_mol_gas


def get_cosmic_mol_gas_density_Decarli2016():
    data_point_Decarli2016_x = [0.4858, 0.9543, 1.4277, 2.6129, 3.8030] # Decarli,2016ApJ...833...69D, CO and cosmic molecular gas mass density.pdf
    data_point_Decarli2016_xmin = np.array([0.2713, 0.6950, 1.0059, 2.0088, 3.0115])
    data_point_Decarli2016_xmax = np.array([0.6309, 1.1744, 1.7387, 3.1080, 4.4771])
    data_point_Decarli2016_ymin = 10**np.array([6.56, 6.83, 7.53, 7.69, 5.53])
    data_point_Decarli2016_ymax = 10**np.array([7.76, 7.73, 8.09, 8.28, 7.58])
    z = {}
    z['center'] = data_point_Decarli2016_x
    z['lower'] = data_point_Decarli2016_xmin
    z['upper'] = data_point_Decarli2016_xmax
    rho_mol_gas = {}
    rho_mol_gas['center'] = (data_point_Decarli2016_ymin + data_point_Decarli2016_ymax) / 2.0
    rho_mol_gas['lower'] = data_point_Decarli2016_ymin
    rho_mol_gas['upper'] = data_point_Decarli2016_ymax
    return z, rho_mol_gas


def get_cosmic_mol_gas_density_Riechers2018():
    data_point_Riechers2018_xmin = np.array([1.95, 1.95, 2.03, 4.90, 4.90]) # COLDZ: CO LUMINOSITY FUNCTION AND COLD GAS HISTORY OF THE UNIVERSE
    data_point_Riechers2018_xmax = np.array([2.85, 2.72, 2.85, 6.70, 6.70])
    data_point_Riechers2018_ymin = np.array([1.1, 0.95, 0.30, 0.14, np.nan])*1e7
    data_point_Riechers2018_ymax = np.array([5.6, 10.9, 7.3, 1.1, 4.0])*1e7 # in their paper the last redshift bin has an error as large as 4.0e7
    #data_point_Riechers2018_x = (data_point_Riechers2018_xmin + data_point_Riechers2018_xmax) / 2.0 # assume αCO = 3.6 Msun (K kms−1 pc2)−1 for all the sources in the sample
    #data_point_Riechers2018_y = (data_point_Riechers2018_ymin + data_point_Riechers2018_ymax) / 2.0 # assume αCO = 3.6 Msun (K kms−1 pc2)−1 for all the sources in the sample
    #data_point_Riechers2018_yerr = np.abs(data_point_Riechers2018_ymin - data_point_Riechers2018_ymax) / 2.0
    z = {}
    z['center'] = (data_point_Riechers2018_xmin + data_point_Riechers2018_xmax) / 2.0
    z['lower'] = data_point_Riechers2018_xmin
    z['upper'] = data_point_Riechers2018_xmax
    rho_mol_gas = {}
    rho_mol_gas['center'] = (data_point_Riechers2018_ymin + data_point_Riechers2018_ymax) / 2.0
    rho_mol_gas['lower'] = data_point_Riechers2018_ymin
    rho_mol_gas['upper'] = data_point_Riechers2018_ymax
    return z, rho_mol_gas


def get_cosmic_mol_gas_density_Decarli2019():
    # -----
    # ----- <20190722>
    # ----- now from https://arxiv.org/pdf/1903.09164.pdf
    # ----- taking Table A.3 "Reference estimate" --  on the whole sample, and assuming intermediate CO excitation (Daddi et al. 2015) as well as the estimates for the secure sources only
    # ----- 
    z1_1sig_lo, z1_1sig_hi, z1_2sig_lo, z1_2sig_hi = 5.89, 6.80, 5.40, 7.01
    z2_1sig_lo, z2_1sig_hi, z2_2sig_lo, z2_2sig_hi = 7.74, 7.96, 7.63, 8.05
    z3_1sig_lo, z3_1sig_hi, z3_2sig_lo, z3_2sig_hi = 7.50, 7.96, 7.26, 8.10
    z4_1sig_lo, z4_1sig_hi, z4_2sig_lo, z4_2sig_hi = 7.20, 7.62, 6.97, 7.77
    data_point_Decarli2019_xmin = np.array([0.002877, 1.006, 2.008, 3.011]) # ASPECS
    data_point_Decarli2019_xmax = np.array([0.369, 1.738, 3.107, 4.475])
    data_point_Decarli2019_1sig_lo = np.array([10**z1_1sig_lo, 10**z2_1sig_lo, 10**z3_1sig_lo, 10**z4_1sig_lo])
    data_point_Decarli2019_1sig_hi = np.array([10**z1_1sig_hi, 10**z2_1sig_hi, 10**z3_1sig_hi, 10**z4_1sig_hi])
    data_point_Decarli2019_2sig_lo = np.array([10**z1_2sig_lo, 10**z2_2sig_lo, 10**z3_2sig_lo, 10**z4_2sig_lo])
    data_point_Decarli2019_2sig_hi = np.array([10**z1_2sig_hi, 10**z2_2sig_hi, 10**z3_2sig_hi, 10**z4_2sig_hi])
    z = {}
    z['center'] = (data_point_Decarli2019_xmin + data_point_Decarli2019_xmax) / 2.0
    z['lower'] = data_point_Decarli2019_xmin
    z['upper'] = data_point_Decarli2019_xmax
    rho_mol_gas = {}
    rho_mol_gas['center'] = (data_point_Decarli2019_1sig_lo + data_point_Decarli2019_1sig_hi) / 2.0
    rho_mol_gas['lower'] = data_point_Decarli2019_1sig_lo
    rho_mol_gas['upper'] = data_point_Decarli2019_1sig_hi
    rho_mol_gas['lower_2sigma'] = data_point_Decarli2019_2sig_lo
    rho_mol_gas['upper_2sigma'] = data_point_Decarli2019_2sig_hi
    return z, rho_mol_gas




# 
# demo plot 
# 
def plot_cosmic_mol_gas_density():
    # 
    # 
    import matplotlib as mpl
    import matplotlib.pyplot as plt
    # 
    # 
    fig = plt.figure(figsize=(6.8,4.8))
    fig.subplots_adjust(left=0.15, right=0.95, bottom=0.105, top=0.885)
    ax = fig.add_subplot(111)
    ax.set_xlabel('Redshift', fontsize=16, labelpad=1)
    ax.set_ylabel(r'$\rho_{\mathrm{mol\,gas}}$ [$\mathrm{M_{\odot}\,Mpc^{-3}}$]', fontsize=17, labelpad=15)
    ax.tick_params(axis='both', labelsize=14)
    ax.tick_params(direction='in', axis='both', which='both')
    ax.tick_params(top=False, right=True, which='both') # top='on', right='on' -- deprecated -- use True/False instead
    ax_tick_locations = np.array([0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0])
    ax.set_xticks(ax_tick_locations)
    ax.set_xlim([-0.3, np.max(ax_tick_locations)])
    ax.set_ylim([1e4, 2e9])
    ax.set_yscale('log')
    ax.grid(True, ls='--', lw=0.25)
    # 
    z, y = get_cosmic_mol_gas_density_Decarli2016()
    for i in range(len(z['center'])):
        if not np.isnan(y['lower'][i]):
            plot_obj_data_1a = ax.fill_between([z['lower'][i], z['upper'][i]], 
                                               [y['lower'][i], y['lower'][i]], 
                                               [y['upper'][i], y['upper'][i]], 
                                               color='#a3e2bc', alpha=0.4, zorder=11) # light green color
    # 
    # 
    z, y = get_cosmic_mol_gas_density_Riechers2018()
    for i in range(len(z['center'])):
        if not np.isnan(y['lower'][i]):
            plot_obj_data_2a = ax.fill_between([z['lower'][i], z['upper'][i]], 
                                               [y['lower'][i], y['lower'][i]], 
                                               [y['upper'][i], y['upper'][i]], 
                                               color='#1e90ff', alpha=0.2, zorder=11) # blue color
        else:
            plot_obj_data_2b = ax.plot([z['lower'][i], z['upper'][i]], 
                                       [y['upper'][i], y['upper'][i]],
                                       color='#1e90ff', alpha=0.25, lw=5, zorder=11) # blue color
            plot_obj_data_2c = plt.arrow(np.mean([z['lower'][i], z['upper'][i]]), 
                                         np.mean([y['upper'][i], y['upper'][i]]),
                                         0.0, 
                                         -np.mean([y['upper'][i], y['upper'][i]]) * 0.6, 
                                         head_length=np.mean([y['upper'][i], y['upper'][i]]) * 0.1, 
                                         head_width=0.2, length_includes_head=True, head_starts_at_zero=False, 
                                         transform=ax.transData, 
                                         color='#1e90ff', alpha=0.25, lw=3, zorder=11) # blue color
    # 
    z, y = get_cosmic_mol_gas_density_Decarli2019()
    for i in range(len(z['center'])):
        if not np.isnan(y['lower'][i]):
            plot_obj_data_3a = ax.fill_between([z['lower'][i], z['upper'][i]], 
                                               [y['lower'][i], y['lower'][i]], 
                                               [y['upper'][i], y['upper'][i]], 
                                               color='#ffa8a8', alpha=0.6, zorder=13) # light red color
        if not np.isnan(y['lower_2sigma'][i]):
            plot_obj_data_3b = ax.fill_between([z['lower'][i], z['upper'][i]], 
                                               [y['lower_2sigma'][i], y['lower_2sigma'][i]], 
                                               [y['upper_2sigma'][i], y['upper_2sigma'][i]], 
                                               color='#ffb9b9', alpha=0.3, zorder=12) # lighter red color
    # 
    # 
    z_A3COSMOS, rho_mol_gas_A3COSMOS = get_cosmic_mol_gas_density_A3COSMOS()
    z_Tacconi2018, rho_mol_gas_Tacconi2018 = get_cosmic_mol_gas_density_Tacconi2018()
    z_Scoville2017, rho_mol_gas_Scoville2017 = get_cosmic_mol_gas_density_Scoville2017()
    plot_obj_Scoville2017 = ax.plot(z_Scoville2017, rho_mol_gas_Scoville2017, dashes=(2,1), color='#f383d7', linewidth=3.5, alpha=0.6, label='__none__', zorder=15) # pink color
    plot_obj_Tacconi2018 = ax.plot(z_Tacconi2018, rho_mol_gas_Tacconi2018, dashes=(5,1), color='#3c9254', linewidth=3.5, alpha=0.6, label='__none__', zorder=15) # green color
    plot_obj_A3COSMOS = ax.plot(z_A3COSMOS, rho_mol_gas_A3COSMOS, linestyle='solid', color='#fd7b0f', linewidth=4.8, alpha=0.8, label='__none__', zorder=15) # a3cosmos red color # 20191020 -> color changed from '#ff1e1f' to '#fd7b0f' (orangered), alpha changed from 0.8 to 0.85.
    plot_label_Scoville2017 = r'$\rho_{M_{\mathrm{mol\,gas}}}$ (Scoville+2017 Eq.)'
    plot_label_Tacconi2018 = r'$\rho_{M_{\mathrm{mol\,gas}}}$ (Tacconi+2018 Eq.)'
    plot_label_A3COSMOS = r'$\rho_{M_{\mathrm{mol\,gas}}}$ (D.Liu+2019b Eq.11)' # \; [\mathrm{M_{\odot}}]
    plot_obj_Mgas_list = [  plot_obj_A3COSMOS[0],
                            plot_obj_Tacconi2018[0],
                            plot_obj_Scoville2017[0],
                         ]
    plot_label_Mgas_list = [  plot_label_A3COSMOS,
                              plot_label_Tacconi2018,
                              plot_label_Scoville2017,
                           ]
    # 
    # 
    # legend
    plot_label_data_1 = r'Decarli+2016'
    plot_label_data_2 = r'Riechers+2019 (COLDz)'
    plot_label_data_3 = r'Decarli+2019 (ASPECS)'
    # 
    plot_obj_data_list = [  \
                            plot_obj_data_1a, 
                            plot_obj_data_2a,
                            plot_obj_data_3a,
                         ]
    plot_label_data_list = [  \
                              plot_label_data_1,
                              plot_label_data_2,
                              plot_label_data_3,
                           ]
    
    plot_legend_1 = plt.legend(plot_obj_data_list, plot_label_data_list, loc='lower left', labelspacing=0.12, borderaxespad=0.25, framealpha=0.5, fontsize=11)
    plot_legend_2 = plt.legend(plot_obj_Mgas_list, plot_label_Mgas_list, loc='lower right', labelspacing=0.12, borderaxespad=0.25, framealpha=0.5, fontsize=11)
    plt.gca().add_artist(plot_legend_1)
    # 
    # 
    # show
    plt.show()
     



# 
# main
# 
if __name__ == '__main__':
    # 
    plot_cosmic_mol_gas_density()






import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'a3cosmos_gas_evolution', 'Common_Python_Code'))

import numpy as np
import pytest
from calc_fmol import (
    calc_fmol_from_metalZ_following_Krumholz2009,
    calc_fmol_from_metalZ_following_Dave2016,
    calc_fmol_from_metalZ_following_Popping2014,
)


class TestCalcFmol:

    def test_krumholz2009_scalar(self):
        metalZOH = 8.67
        result = calc_fmol_from_metalZ_following_Krumholz2009(metalZOH)
        assert 0.0 <= result <= 1.0

    def test_krumholz2009_array(self):
        metalZOH = np.array([8.0, 8.5, 8.67, 9.0])
        result = calc_fmol_from_metalZ_following_Krumholz2009(metalZOH)
        assert len(result) == 4
        assert np.all(result >= 0.0) and np.all(result <= 1.0)

    def test_krumholz2009_low_metallicity(self):
        metalZOH = 7.5
        result = calc_fmol_from_metalZ_following_Krumholz2009(metalZOH)
        assert 0.0 <= result <= 1.0

    def test_krumholz2009_high_metallicity(self):
        metalZOH = 9.5
        result = calc_fmol_from_metalZ_following_Krumholz2009(metalZOH)
        assert 0.0 <= result <= 1.0

    def test_dave2016_scalar(self):
        metalZOH = 8.67
        result = calc_fmol_from_metalZ_following_Dave2016(metalZOH)
        assert 0.0 <= result <= 1.0

    def test_dave2016_array(self):
        metalZOH = np.array([8.0, 8.5, 8.67, 9.0])
        result = calc_fmol_from_metalZ_following_Dave2016(metalZOH)
        assert len(result) == 4

    def test_dave2016_low_metallicity(self):
        metalZOH = 7.5
        result = calc_fmol_from_metalZ_following_Dave2016(metalZOH)
        assert result > -2.0

    def test_popping2014_scalar(self):
        metalZOH = 8.67
        U_MW = 1.0
        result = calc_fmol_from_metalZ_following_Popping2014(np.array([metalZOH]), U_MW=np.array([U_MW]))
        assert result[0] >= 0.0 and result[0] <= 1.0

    def test_popping2014_array(self):
        metalZOH = np.array([8.0, 8.5, 8.67, 9.0])
        U_MW = np.ones_like(metalZOH)
        result = calc_fmol_from_metalZ_following_Popping2014(metalZOH, U_MW=U_MW)
        assert len(result) == 4

    def test_fmol_increases_with_metallicity(self):
        metalZ_low = 8.0
        metalZ_high = 9.0
        result_low = calc_fmol_from_metalZ_following_Krumholz2009(metalZ_low)
        result_high = calc_fmol_from_metalZ_following_Krumholz2009(metalZ_high)
        assert result_high > result_low



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'a3cosmos_gas_evolution', 'Common_Python_Code'))

import numpy as np
import pytest


class TestCalcDustOpacityKappa:

    def test_kappa_above_700um(self):
        import calc_dust_opacity_kappa
        lambda_um = 850.0
        beta = 1.68
        kappa = 0.596 * (lambda_um / 700.0)**(-beta)
        assert kappa > 0

    def test_kappa_below_700um(self):
        import calc_dust_opacity_kappa
        lambda_um = 500.0
        beta = 2.0
        kappa = 0.596 * (lambda_um / 700.0)**(-beta)
        assert kappa > 0

    def test_kappa_at_700um_transition(self):
        lambda_um = 700.0
        beta_high = 1.68
        beta_low = 2.0
        kappa_high = 0.596 * (lambda_um / 700.0)**(-beta_high)
        kappa_low = 0.596 * (lambda_um / 700.0)**(-beta_low)
        assert np.isclose(kappa_high, kappa_low)

    def test_kappa_wavelength_dependence(self):
        lambda_1 = 500.0
        lambda_2 = 1000.0
        beta = 2.0
        kappa_1 = 0.596 * (lambda_1 / 700.0)**(-beta)
        kappa_2 = 0.596 * (lambda_2 / 700.0)**(-beta)
        assert kappa_1 > kappa_2



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'a3cosmos_gas_evolution', 'Common_Python_Code'))

import numpy as np
import pytest
from calc_gas_depletion_time import (
    calc_gas_depletion_time_Scoville2017,
    calc_gas_depletion_time_Tacconi2018,
    calc_gas_depletion_time_A3COSMOS,
)


class TestCalcGasDepletionTime:

    def test_scoville2017_scalar(self):
        result = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_scoville2017_z0(self):
        result = calc_gas_depletion_time_Scoville2017(z=0.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_scoville2017_high_z(self):
        result = calc_gas_depletion_time_Scoville2017(z=3.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_scoville2017_different_mass(self):
        result_low = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=9.0, DeltaMS=0.0)
        result_high = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=11.0, DeltaMS=0.0)
        assert result_low > 0 and result_high > 0

    def test_scoville2017_different_DeltaMS(self):
        result_D0 = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=10.5, DeltaMS=0.0)
        result_D1 = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=10.5, DeltaMS=1.0)
        assert result_D1 < result_D0

    def test_tacconi2018_scalar(self):
        result = calc_gas_depletion_time_Tacconi2018(z=1.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_tacconi2018_z0(self):
        result = calc_gas_depletion_time_Tacconi2018(z=0.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_tacconi2018_high_z(self):
        result = calc_gas_depletion_time_Tacconi2018(z=3.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_a3cosmos_z(self):
        result = calc_gas_depletion_time_A3COSMOS(z=1.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_a3cosmos_cosmic_age(self):
        result = calc_gas_depletion_time_A3COSMOS(cosmic_age=5.0, lgMstar=10.5, DeltaMS=0.0)
        assert result > 0

    def test_a3cosmos_z_and_cosmic_age_consistent(self):
        import apply_cosmology
        cosmo = apply_cosmology.cosmo
        z = 1.0
        cosmic_age = cosmo.age(z).value
        result_z = calc_gas_depletion_time_A3COSMOS(z=z, lgMstar=10.5, DeltaMS=0.0)
        result_age = calc_gas_depletion_time_A3COSMOS(cosmic_age=cosmic_age, lgMstar=10.5, DeltaMS=0.0)
        assert np.isclose(result_z, result_age)

    def test_tauDepl_decreases_with_z(self):
        result_z0 = calc_gas_depletion_time_Scoville2017(z=0.5, lgMstar=10.5, DeltaMS=0.0)
        result_z2 = calc_gas_depletion_time_Scoville2017(z=2.0, lgMstar=10.5, DeltaMS=0.0)
        assert result_z2 < result_z0

    def test_tauDepl_increases_with_DeltaMS(self):
        result_D0 = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=10.5, DeltaMS=0.0)
        result_D1 = calc_gas_depletion_time_Scoville2017(z=1.0, lgMstar=10.5, DeltaMS=1.0)
        assert result_D1 < result_D0



import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'a3cosmos_gas_evolution', 'Common_Python_Code'))

import numpy as np
import pytest
from astropy import units as u


class TestCalcPlanckLaw:

    def test_blackbody_nu_at_25K(self):
        from calc_planck_law import blackbody_nu
        wavelength = 100.0 * u.um
        T = 25.0 * u.K
        result = blackbody_nu(wavelength, T)
        assert result.value >= 0

    def test_blackbody_lambda_at_25K(self):
        from calc_planck_law import blackbody_lambda
        wavelength = 100.0 * u.um
        T = 25.0 * u.K
        result = blackbody_lambda(wavelength, T)
        assert result.value >= 0

    def test_blackbody_nu_at_50K(self):
        from calc_planck_law import blackbody_nu
        wavelength = 100.0 * u.um
        T = 50.0 * u.K
        result = blackbody_nu(wavelength, T)
        assert result.value >= 0

    def test_blackbody_returns_positive(self):
        from calc_planck_law import blackbody_nu, blackbody_lambda
        wavelengths = np.array([50.0, 100.0, 200.0]) * u.um
        T = 25.0 * u.K
        for w in wavelengths:
            result_nu = blackbody_nu(w, T)
            result_lambda = blackbody_lambda(w, T)
            assert result_nu.value >= 0
            assert result_lambda.value >= 0

    def test_blackbody_high_temp_greater_than_low_temp(self):
        from calc_planck_law import blackbody_nu
        wavelength = 100.0 * u.um
        T_low = 20.0 * u.K
        T_high = 30.0 * u.K
        result_low = blackbody_nu(wavelength, T_low)
        result_high = blackbody_nu(wavelength, T_high)
        assert result_high.value > result_low.value



#!/usr/bin/env python
#

from __future__ import print_function

import os, sys, re, json, time, astropy
import numpy as np
#from astropy.table import Table, Column, hstack
from astropy import units as u
from copy import copy

if not (os.path.dirname(os.path.abspath(__file__)) in sys.path): sys.path.append(os.path.dirname(os.path.abspath(__file__)))
import apply_cosmology
#cosmo = apply_cosmology.cosmo
cosmo = apply_cosmology.apply_cosmology(70, 0.3, 0.7)
print('cosmo', cosmo)

if sys.version_info.major >= 3:
    long = int
else:
    pass




def calc_cosmic_comoving_volume_1(z_edge_1, z_edge_2, obs_area_arcmin2):
    if type(obs_area_arcmin2) is u.quantity.Quantity:
        obs_area = obs_area_arcmin2
    else:
        obs_area = obs_area_arcmin2 * u.arcmin**2
    #comoving_z_list = np.linspace(z_edges[i], z_edges[i+1], num=100, endpoint=True)
    #comoving_volume = np.sum((cosmo.comoving_volume(comoving_z_list[1:]) - cosmo.comoving_volume(comoving_z_list[0:-1])) / (4.0*np.pi*u.steradian) * obs_area.to(u.steradian))
    #print('comoving_volume = %e [%s]'%(comoving_volume.value, comoving_volume.unit))
    comoving_volume = ((cosmo.comoving_volume(z_edge_2) - cosmo.comoving_volume(z_edge_1)) / (4.0*np.pi*u.steradian) * obs_area.to(u.steradian))
    print('comoving_volume = %e [%s]'%(comoving_volume.value, comoving_volume.unit))
    return comoving_volume.value



def calc_cosmic_comoving_volume_2(z_edge_1, z_edge_2, obs_area_arcmin2):
    if type(obs_area_arcmin2) is u.quantity.Quantity:
        obs_area = obs_area_arcmin2
    else:
        obs_area = obs_area_arcmin2 * u.arcmin**2
    differntial_z_list = np.linspace(z_edge_1, z_edge_2, num=10, endpoint=True)
    comoving_volume = np.sum((cosmo.differential_comoving_volume(differntial_z_list[1:]) * np.diff(differntial_z_list) * obs_area.to(u.steradian)))
    print('comoving_volume = %e [%s]'%(comoving_volume.value, comoving_volume.unit))
    ##print(cosmo.de_density_scale(z)) # should be 1
    ##print(cosmo._Ogamma0, cosmo._Onu0)
    ##print(cosmo.efunc(z), np.sqrt(0.27*(1.+z)**3 + 0*(1.+z)**2 + 0.73) ) # checked consistent
    ##print(cosmo._hubble_distance, 2.997902458e5/70 ) # checked consistent
    #sys_lumdist_output = subprocess.getoutput("/Users/dzliu/Cloud/Github/Crab.Toolkit.PdBI/bin/lumdist -h0 70 -verbose %s | grep 'lumdist d_L=' | sed -e 's/=/ /g'"%(z))
    ##print(cosmo.angular_diameter_distance(z), sys_lumdist_output.split()[8], '(z = %s)'%(z), sys_lumdist_output ) # 
    #dH_astropy = cosmo._hubble_distance
    #Ez_astropy = cosmo.efunc(z)
    #dA_astropy = cosmo.angular_diameter_distance(z)
    #dH_dzliu = 2.997902458e5/70
    #Ez_dzliu = np.sqrt(0.27*(1.+z)**3 + 0*(1.+z)**2 + 0.73)
    #dA_dzliu = float(sys_lumdist_output.split()[8])
    #print(dH_astropy/Ez_astropy*dA_astropy, dH_dzliu/Ez_dzliu*dA_dzliu) # chekced consistent
    #zp1 = 1.0 + z
    #print(cosmo.differential_comoving_volume(z), dH_astropy*((zp1*dA_astropy)**2)/Ez_astropy, dH_dzliu/Ez_dzliu*dA_dzliu**2*zp1**2) # chekced consistent
    # z = 2.0 - 2.5, area = 0.0006092348395183178 steradian, dVc = 43627725623.05944, 
    # 43627725623.05944 * 0.25 / 10
    return comoving_volume.value



if __name__ == '__main__':
    
    obs_area = 1.5546582999901375*u.deg*u.deg
    print('obs_area = %s [%s]'%(obs_area.to(u.arcmin*u.arcmin).value, obs_area.to(u.arcmin*u.arcmin).unit))
    print('obs_area = %s [%s]'%(obs_area.to(u.steradian).value, obs_area.to(u.steradian).unit))
    #print(type(obs_area))
    
    z_edges = [0.02, 0.25, 0.50, 0.75, 1.00, 1.5, 2.0, 2.5, 3.0, 4.0, 5.0, 6.0]
    
    # 
    # loop z bin
    for i in range(len(z_edges)-1):
        # 
        print('z %s - %s, cosmic age %.2f - %.2f, time interval %.2f'%(\
                z_edges[i], 
                z_edges[i+1], 
                cosmo.age(z_edges[i]).to('Gyr').value, 
                cosmo.age(z_edges[i+1]).to('Gyr').value, 
                cosmo.age(z_edges[i]).to('Gyr').value - cosmo.age(z_edges[i+1]).to('Gyr').value
                )
             )
        # 
        z = (z_edges[i]+z_edges[i+1])/2.0
        # 
        calc_cosmic_comoving_volume_1(z_edges[i], z_edges[i+1], obs_area)
        calc_cosmic_comoving_volume_2(z_edges[i], z_edges[i+1], obs_area)
        # 
        # 
        # 
    


